# Detection of Potential Geothermal Exploration Sites from Hyperspectral Images via Deep Learning
# Author: Ebubekir Demir <edemir@mines.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import numpy as np
import argparse
import time

start = time.time()
from datetime import timedelta


def unique(data):
    uniquev, counts = np.unique(data, return_counts=True)
    print(np.array([uniquev, counts]).T)


def apply(file, prediction):
    xai0 = np.load(file + "-0-pre.npy")
    xai1 = np.load(file + "-1-pre.npy")
    xai2 = np.load(file + "-2-pre.npy")
    xai3 = np.load(file + "-3-pre.npy")
    xai4 = np.load(file + "-4-pre.npy")
    xai5 = np.load(file + "-5-pre.npy")
    pre = np.load(prediction)

    data_01 = np.max(np.stack((xai0, xai1)), axis=0)
    data_23 = np.max(np.stack((xai2, xai3)), axis=0)
    data_45 = np.max(np.stack((xai4, xai5)), axis=0)

    data_0123 = np.max(np.stack((data_01, data_23)), axis=0)
    data_0145 = np.max(np.stack((data_01, data_45)), axis=0)
    data_2345 = np.max(np.stack((data_23, data_45)), axis=0)

    data_01234 = np.max(np.stack((xai4, data_0123)), axis=0)
    data_01235 = np.max(np.stack((xai5, data_0123)), axis=0)
    data_01245 = np.max(np.stack((xai2, data_0145)), axis=0)
    data_01345 = np.max(np.stack((xai3, data_0145)), axis=0)
    data_02345 = np.max(np.stack((xai0, data_2345)), axis=0)
    data_12345 = np.max(np.stack((xai1, data_2345)), axis=0)

    res = np.empty(pre.shape)

    res[xai0 >= data_12345] = 0
    res[xai1 >= data_02345] = 1
    res[xai2 >= data_01345] = 2
    res[xai3 >= data_01245] = 3
    res[xai4 >= data_01235] = 4
    res[xai5 >= data_01234] = 5

    res[np.isnan(pre)] = np.nan

    unique(res)
    np.save(file + "-dmb.npy", res)
    print("Saved", file + "-dmb.npy")


if __name__ == '__main__':
    ap = argparse.ArgumentParser()
    ap.add_argument("-f", "--file", required=True, help="model file to read(h5)", default=None)
    ap.add_argument("-p", "--prediction", required=True, help="model file to read(h5)", default=None)
    args = vars(ap.parse_args())
    file = args["file"]
    pre = args["prediction"]
    apply(file, pre)

end = time.time()
td = timedelta(seconds=int(end - start))
print("Total processing time", td)


