# Detection of Potential Geothermal Exploration Sites from Hyperspectral Images via Deep Learning
# Author: Ebubekir Demir <edemir@mines.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import argparse

import numpy as np
from tqdm import tqdm


def getData(layers, kernel_pixels):
    padding = int(kernel_pixels / 2)
    data = []
    for layer in layers:
        npa = np.load(layer)
        npa = np.pad(npa, (padding,), 'edge')
        npa = np.array(npa, dtype=np.float32)
        data.append(npa)
    return data


def filterData(data, gnd_layer, kernel_pixels, resultfile):
    print(gnd_layer, kernel_pixels, resultfile)
    lbl = []
    gnd = np.load(gnd_layer)
    print(gnd.shape)
    ls = np.array(np.where(gnd >= 0))
    print(ls.shape)
    choices = np.random.choice(len(ls.T), len(ls.T), replace=False)
    for c in tqdm(choices):
        z = ls[0][c]
        y = ls[1][c]
        x = ls[2][c]
        batch = []
        for band in data:
            batch.append(np.copy(band[z:z + kernel_pixels, y:y + kernel_pixels, x:x + kernel_pixels]))
        vl = np.array(batch)
        if not np.isnan(vl).any():
            lbl.append([z, y, x])
    res = np.asarray(lbl, dtype=np.int32)
    print(res.shape, res.dtype)
    np.save(resultfile, res)


if __name__ == '__main__':
    ap = argparse.ArgumentParser()
    ap.add_argument("-l", "--layers", required=True, help="input layers", nargs='+', default=[])
    ap.add_argument("-g", "--ground", required=True, help="ground truth", default=None)
    ap.add_argument("-k", "--kernelpixels", required=True, help="kernel pixels size", default=0)
    ap.add_argument("-r", "--result", required=True, help="result npy file", default=None)

    args = vars(ap.parse_args())
    lyr = args["layers"]
    gnd = args["ground"]
    res = args["result"]
    kernel = int(args["kernelpixels"])

    data = getData(lyr, kernel)
    filterData(data, gnd, kernel, res)
