# Detection of Potential Geothermal Exploration Sites from Hyperspectral Images via Deep Learning
# Author: Ebubekir Demir <edemir@mines.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import os

import numpy as np
import argparse
import time

start = time.time()
from datetime import timedelta


def unique(data):
    uniquev, counts = np.unique(data, return_counts=True)
    print(np.array([uniquev, counts]).T)


def apply(xairesult, result, ground, prediction):
    xai = np.load(xairesult)
    gnd = np.load(ground)
    pre = np.load(prediction)

    res = np.copy(xai)
    res[gnd != 1] = np.nan
    res[pre != 1] = np.nan
    unique(res)
    np.save(result, res)
    print("Saved", result)


if __name__ == '__main__':
    ap = argparse.ArgumentParser()
    ap.add_argument("-d", "--directory", required=True, help="model file to read(h5)", default=None)
    #ap.add_argument("-x", "--xai", required=True, help="model file to read(h5)", default=None)
    #ap.add_argument("-r", "--result", required=True, help="kernel pixels size", default=0)
    ap.add_argument("-g", "--ground", required=True, help="model file to read(h5)", default=None)
    ap.add_argument("-p", "--prediction", required=True, help="result file to write(npy)", default=None)
    args = vars(ap.parse_args())
    #xairesult = args["xai"]
    #result = args["result"]
    dir = args["directory"]
    ground = args["ground"]
    prediction = args["prediction"]
    for filename in os.listdir(dir):
        if "-0.npy" in filename or "-1.npy" in filename or "-2.npy" in filename or "-3.npy" in filename or "-4.npy" in filename or "-5.npy" in filename:
            fl =  filename.split(".")
            xairesult = os.path.join(dir, filename)
            result = os.path.join(dir, fl[0] + "-pre.npy")
            if not os.path.isfile(result):
                print(xairesult)
                print(result)
                apply(xairesult, result, ground, prediction)

end = time.time()
td = timedelta(seconds=int(end - start))
print("Total processing time", td)
