# Detection of Potential Geothermal Exploration Sites from Hyperspectral Images via Deep Learning
# Author: Ebubekir Demir <edemir@mines.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import numpy as np
import pandas as pd
import argparse
from osgeo import gdal
from tqdm import tqdm


def exporttoLF(rasterfile, npyfile, csvcolumnname, exportfile, zref=850, width=100):
    ds = gdal.Open(rasterfile)
    pre = np.load(npyfile)
    gt = ds.GetGeoTransform()
    xref = gt[0]
    yref = gt[3]
    size = gt[1]
    center = size / 2
    X = []
    Y = []
    Z = []
    Pr = []
    for z in tqdm(range(pre.shape[0])):
        for y in range(pre.shape[1]):
            for x in range(pre.shape[2]):
                xval = xref + (x * size) + center
                yval = yref - (y * size) - center
                zval = zref - (z * size)
                X.append(xval)
                Y.append(yval)
                Z.append(zval)
                Pr.append(pre[z][y][x])

    dict_val = {"X": np.array(X), "Y": np.array(Y), "Z": np.array(Z), csvcolumnname: np.array(Pr)}
    df_val = pd.DataFrame(dict_val)
    df_val.to_csv(exportfile, index=True, index_label="Id")
    print("Result CSV file exported to", exportfile)


if __name__ == '__main__':
    ap = argparse.ArgumentParser()
    ap.add_argument("-r", "--raster", required=True, help="raster file", default=None)
    ap.add_argument("-f", "--file", required=True, help="file to export", default=None)
    ap.add_argument("-n", "--name", required=True, help="name of column", default=None)
    ap.add_argument("-e", "--export", required=True, help="export csv filename", default=None)
    args = vars(ap.parse_args())
    raster = args["raster"]
    file = args["file"]
    name = args["name"]
    export = args["export"]

    exporttoLF(raster, file, name, export)
