# Detection of Potential Geothermal Exploration Sites from Hyperspectral Images via Deep Learning
# Author: Ebubekir Demir <edemir@mines.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import math

import numpy as np
import pandas as pd
from numpy.linalg import LinAlgError

import constants as ct
import argparse
import os

from osgeo import gdal, ogr, osr
from tqdm import tqdm


def createprojectionintersection(refrasterfile, rasterfile, resultfile, zlenght=12300):
    # project rasterfile
    dsp = gdal.Open(rasterfile)
    gtp = dsp.GetGeoTransform()
    band = dsp.GetRasterBand(1)
    data = band.ReadAsArray()
    data[data < 0] = 0
    xrefp = gtp[0]
    yrefp = gtp[3]
    sizep = gtp[1]
    print(dsp.GetProjection())
    # reference raster file
    ds = gdal.Open(refrasterfile)
    xsize = ds.RasterXSize
    ysize = ds.RasterYSize
    gt = ds.GetGeoTransform()
    xref = gt[0]
    yref = gt[3]
    size = gt[1]
    center = size / 2
    print(ds.GetProjection())
    zlenght = zlenght * ct.FEET
    zsize = int(zlenght / size) + 1
    result = np.zeros((zsize, ysize, xsize))
    count = 0
    for y in tqdm(range(result.shape[1])):
        for x in range(result.shape[2]):
            xmin = xref + (x * size) - center
            xmax = xref + (x * size) + center
            ymin = yref - (y * size) - center
            ymax = yref - (y * size) + center
            xminp = int((xmin - xrefp) / sizep)
            xmaxp = int((xmax - xrefp) / sizep) + 1
            yminp = int((yrefp - ymax) / sizep)
            ymaxp = int((yrefp - ymin) / sizep) + 1
            if xminp < 0:
                xminp = 0
            if xmaxp < 0:
                xmaxp = 0
            if yminp < 0:
                yminp = 0
            if ymaxp < 0:
                ymaxp = 0
            datap = data[yminp:ymaxp, xminp:xmaxp]
            sum = np.nanmean(datap)
            if sum != 0:
                result[:,y,x] = sum
                count = count + 1
    print(count)
    np.save(resultfile, result)


if __name__ == '__main__':
    ap = argparse.ArgumentParser()
    ap.add_argument("-f", "--reference", required=True, help="reference raster file")
    ap.add_argument("-r", "--raster", required=True, help="raster file")
    ap.add_argument("-o", "--output", required=True, help="output file (.npy)")
    args = vars(ap.parse_args())
    gnd = args["reference"]
    rst = args["raster"]
    result = args["output"]
    createprojectionintersection(gnd, rst, result)
