# Detection of Potential Geothermal Exploration Sites from Hyperspectral Images via Deep Learning
# Author: Ebubekir Demir <edemir@mines.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.

import numpy as np
import tensorflow as tf
import argparse
import time
import pandas as pd
import gc

start = time.time()
from datetime import timedelta
from tqdm import tqdm
from keras.models import load_model
import os

import innvestigate


def xai_investiagte(layers, modelfile, kernel_pixels, resultfile, xaimethod, xsize=493, ysize=493, zsize=188, limit=1000):
    padding = int(kernel_pixels / 2)
    data = []
    for layer in layers:
        data.append(np.load(layer))
    print("Creating Model")
    print("NUmber of bands:", len(layers))
    model = load_model(modelfile)
    for i in range(len(data)):
        u_data = data[i]
        data[i] = np.pad(u_data, (padding,), 'edge')

    result = np.empty((len(data), zsize, ysize, xsize))
    result[:] = np.nan
    data = np.array(data, dtype=float)

    desc = "Predicting..." + modelfile
    model_wo_softmax = innvestigate.model_wo_softmax(model)
    try:
        analyzer = innvestigate.create_analyzer(xaimethod, model_wo_softmax)
    except:
        print("ex")
    count = 0
    ct = 0
    for z in tqdm(range(zsize), desc=desc, ascii=False, ncols=75):
    #for z in tqdm(range(1), desc=desc, ascii=False, ncols=75):
        dt = []
        xx = []
        yy = []
        index_arr = np.array(np.where(data[0][0] >= 0))
        total_lenght = len(index_arr.T)
        choices = np.random.choice(total_lenght, total_lenght, replace=False)
        # Define distribution strategy
        strategy = tf.distribute.MirroredStrategy()
        # construct model under distribution strategy scope
        with strategy.scope():
            for c in choices:
                y = index_arr[0][c]
                x = index_arr[1][c]
                batch = []
                for band in data:
                    batch.append(np.copy(band[z:z + kernel_pixels, y:y + kernel_pixels, x:x + kernel_pixels]))
                vl = np.array(batch)
                if not np.isnan(vl).any():
                    xx.append(x)
                    yy.append(y)
                    dt.append(vl)
                    count = count + 1
                    if count == limit:
                        data_final = np.array(dt, dtype=np.float64)
                        a = analyzer.analyze(data_final)
                        weights = np.sum(a, axis=-1)
                        weights = np.sum(weights, axis=-1)
                        weights = np.sum(weights, axis=-1)
                        for i in range(len(weights)):
                            result[:, z, yy[i], xx[i]] = weights[i]
                        dt = []
                        xx = []
                        yy = []
                        count = 0
                        ct = ct + 1
                        print("batch", ct)
            if count != 0:
                data_final = np.array(dt, dtype=np.float64)
                a = analyzer.analyze(data_final)
                weights = np.sum(a, axis=-1)
                weights = np.sum(weights, axis=-1)
                weights = np.sum(weights, axis=-1)
                for i in range(len(weights)):
                    result[:, z, yy[i], xx[i]] = weights[i]
                print("batch end")

    for i in range(len(layers)):
        layer = layers[i]
        ly = layer.split("/")
        dt = result[i, :, :, :]
        np.save(resultfile + "-" + xaimethod + "-" + ly[-1], dt)
    print(result.shape)
    w = np.nansum(result, axis=-1)
    print(w.shape)
    w = np.nansum(w, axis=-1)
    print(w.shape)
    w = np.nansum(w, axis=-1)
    print(w.shape)
    print(w)
    dict_val = {"Layers": np.array(layers), "Weights": w}
    df_val = pd.DataFrame(dict_val)
    df_val.to_csv(resultfile + "-" + xaimethod + ".csv", index=True, index_label="Id")


if __name__ == '__main__':
    ap = argparse.ArgumentParser()
    ap.add_argument("-l", "--layers", required=True, help="help", nargs='+', default=[])
    ap.add_argument("-k", "--kernelpixels", required=True, help="kernel pixels size", default=0)
    ap.add_argument("-m", "--model", required=True, help="model file to read(h5)", default=None)
    ap.add_argument("-r", "--result", required=True, help="result file to write(npy)", default=None)
    args = vars(ap.parse_args())
    lyr = args["layers"]
    kernel = int(args["kernelpixels"])
    model = args["model"]
    rst = args["result"]
    method = "lrp.w_square"
    xai_investiagte(lyr, model, kernel, rst, method)

end = time.time()
td = timedelta(seconds=int(end - start))
print("Total processing time", td)
